﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Optimization.Operators.LCS {
  [Item("ReinitializeOperator", "Description missing")]
  [StorableClass]
  public class ReinitializeOperator : GAssistSpecialStageOperator {

    #region Parameter Properties
    public ILookupParameter<ItemCollection<IDiscretizer>> DiscretizersParameter {
      get { return (ILookupParameter<ItemCollection<IDiscretizer>>)Parameters["Discretizers"]; }
    }
    public ILookupParameter<PercentValue> OneProbabilityParameter {
      get { return (ILookupParameter<PercentValue>)Parameters["OneProbability"]; }
    }
    #endregion

    [StorableConstructor]
    protected ReinitializeOperator(bool deserializing) : base(deserializing) { }
    protected ReinitializeOperator(ReinitializeOperator original, Cloner cloner)
      : base(original, cloner) {
    }
    public ReinitializeOperator()
      : base() {
      Parameters.Add(new LookupParameter<ItemCollection<IDiscretizer>>("Discretizers", ""));
      Parameters.Add(new LookupParameter<PercentValue>("OneProbability", ""));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ReinitializeOperator(this, cloner);
    }

    protected override void ApplySpecialStage(IGAssistIndividual individual, double probability, IRandom random) {
      individual.ApplyReinitialize(random, probability, OneProbabilityParameter.ActualValue.Value, DiscretizersParameter.ActualValue);
    }
  }
}
